@@ -17,7 +17,7 @@
 
 #include "filteractionsendfakedispositiontest.h"
 #include "../filteractions/filteractionsendfakedisposition.h"
-#include <qtest.h>
+#include <QTest>
 #include <pimcommon/minimumcombobox.h>
 
 FilterActionSendFakeDispositionTest::FilterActionSendFakeDispositionTest(QObject *parent)
