@@ -34,7 +34,7 @@ FilterAction *FilterActionRemoveHeader::newAction()
 FilterActionRemoveHeader::FilterActionRemoveHeader(QObject *parent)
     : FilterActionWithStringList(QStringLiteral("remove header"), i18n("Remove Header"), parent)
 {
-    mParameterList << QStringLiteral("")
+    mParameterList << QString()
                    << QStringLiteral("Reply-To")
                    << QStringLiteral("Delivered-To")
                    << QStringLiteral("X-KDE-PR-Message")
@@ -51,7 +51,7 @@ QWidget *FilterActionRemoveHeader::createParamWidget(QWidget *parent) const
     comboBox->setInsertPolicy(QComboBox::InsertAtBottom);
     setParamWidgetValue(comboBox);
 
-    connect(comboBox, static_cast<void (PimCommon::MinimumComboBox::*)(int)>(&PimCommon::MinimumComboBox::currentIndexChanged), this, &FilterActionRemoveHeader::filterActionModified);
+    connect(comboBox, QOverload<int>::of(&PimCommon::MinimumComboBox::currentIndexChanged), this, &FilterActionRemoveHeader::filterActionModified);
     connect(comboBox->lineEdit(), &QLineEdit::textChanged,
             this, &FilterAction::filterActionModified);
 
