@@ -26,7 +26,6 @@
 
 #include <QHBoxLayout>
 #include <QLabel>
-#include <QTextDocument>
 
 using namespace MailCommon;
 
@@ -38,7 +37,7 @@ FilterAction *FilterActionRewriteHeader::newAction()
 FilterActionRewriteHeader::FilterActionRewriteHeader(QObject *parent)
     : FilterActionWithStringList(QStringLiteral("rewrite header"), i18n("Rewrite Header"), parent)
 {
-    mParameterList << QStringLiteral("")
+    mParameterList << QString()
                    << QStringLiteral("Subject")
                    << QStringLiteral("Reply-To")
                    << QStringLiteral("Delivered-To")
@@ -112,7 +111,7 @@ QWidget *FilterActionRewriteHeader::createParamWidget(QWidget *parent) const
     QWidget *widget = new QWidget(parent);
     QHBoxLayout *layout = new QHBoxLayout(widget);
     layout->setSpacing(4);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
 
     PimCommon::MinimumComboBox *comboBox = new PimCommon::MinimumComboBox(widget);
     comboBox->setEditable(true);
@@ -149,7 +148,7 @@ QWidget *FilterActionRewriteHeader::createParamWidget(QWidget *parent) const
 
     setParamWidgetValue(widget);
 
-    connect(comboBox, static_cast<void (PimCommon::MinimumComboBox::*)(int)>(&PimCommon::MinimumComboBox::currentIndexChanged), this, &FilterActionRewriteHeader::filterActionModified);
+    connect(comboBox, QOverload<int>::of(&PimCommon::MinimumComboBox::currentIndexChanged), this, &FilterActionRewriteHeader::filterActionModified);
     connect(comboBox->lineEdit(), &QLineEdit::textChanged,
             this, &FilterAction::filterActionModified);
     connect(regExpLineEdit, &KLineEdit::textChanged, this, &FilterActionRewriteHeader::filterActionModified);
