@@ -110,7 +110,7 @@ FilterActionWidget::FilterActionWidget(QWidget *parent)
     , d(new Private(this))
 {
     QHBoxLayout *mainLayout = new QHBoxLayout(this);
-    mainLayout->setMargin(0);
+    mainLayout->setContentsMargins(0, 0, 0, 0);
     QWidget *widget = new QWidget(this);
     mainLayout->addWidget(widget);
 
@@ -283,7 +283,8 @@ void FilterActionWidgetLister::Private::regenerateActionListFromWidgets()
 
     mActionList->clear();
 
-    foreach (const QWidget *widget, q->widgets()) {
+    const auto lst = q->widgets();
+    for (const QWidget *widget : lst) {
         FilterAction *action = qobject_cast<const FilterActionWidget *>(widget)->action();
         if (action) {
             mActionList->append(action);
