@@ -42,19 +42,7 @@ class Q_DECL_HIDDEN SnippetsManager::Private
 public:
     Private(SnippetsManager *qq, QWidget *parent)
         : q(qq)
-        , mModel(nullptr)
-        , mSelectionModel(nullptr)
-        , mActionCollection(nullptr)
-        , mEditor(nullptr)
-        , mAddSnippetAction(nullptr)
-        , mEditSnippetAction(nullptr)
-        , mDeleteSnippetAction(nullptr)
-        , mAddSnippetGroupAction(nullptr)
-        , mEditSnippetGroupAction(nullptr)
-        , mDeleteSnippetGroupAction(nullptr)
-        , mInsertSnippetAction(nullptr)
         , mParent(parent)
-        , mDirty(false)
     {
     }
 
@@ -78,12 +66,11 @@ public:
     void slotAddNewDndSnippset(const QString &);
 
     void updateActionCollection(const QString &oldName, const QString &newName, const QKeySequence &keySequence, const QString &text);
+    void initializeAction(const QString &newName, const QKeySequence &keySequence, const QString &text);
+    void initializeActionCollection();
 
     QString replaceVariables(const QString &text);
-    QModelIndex createGroup(const QString &groupName);
-    void createSnippet(const QModelIndex &groupIndex, const QString &snippetName, const QString &snippetText, const QString &snippetKeySequence);
 
-    void load();
     void save();
 
     SnippetsManager *q = nullptr;
@@ -92,7 +79,6 @@ public:
     KActionCollection *mActionCollection = nullptr;
     QObject *mEditor = nullptr;
     QByteArray mEditorInsertMethod;
-    QMap<QString, QString> mSavedVariables;
 
     QAction *mAddSnippetAction = nullptr;
     QAction *mEditSnippetAction = nullptr;
@@ -188,7 +174,7 @@ void SnippetsManager::Private::createSnippet(const QString &text)
         mModel->setData(index, dlg->text(), SnippetsModel::TextRole);
         mModel->setData(index, dlg->keySequence().toString(), SnippetsModel::KeySequenceRole);
 
-        updateActionCollection(QString(), dlg->name(), dlg->keySequence(), dlg->text());
+        Q_EMIT mModel->updateActionCollection(QString(), dlg->name(), dlg->keySequence(), dlg->text());
         mDirty = true;
         save();
     }
@@ -240,7 +226,7 @@ void SnippetsManager::Private::editSnippet()
         mModel->setData(index, dlg->text(), SnippetsModel::TextRole);
         mModel->setData(index, dlg->keySequence().toString(), SnippetsModel::KeySequenceRole);
 
-        updateActionCollection(oldSnippetName, dlg->name(), dlg->keySequence(), dlg->text());
+        Q_EMIT mModel->updateActionCollection(oldSnippetName, dlg->name(), dlg->keySequence(), dlg->text());
         mDirty = true;
         save();
     }
@@ -265,7 +251,7 @@ void SnippetsManager::Private::deleteSnippet()
 
     mModel->removeRow(index.row(), currentGroupIndex());
 
-    updateActionCollection(snippetName, QString(), QKeySequence(), QString());
+    Q_EMIT mModel->updateActionCollection(snippetName, QString(), QKeySequence(), QString());
     mDirty = true;
     save();
 }
@@ -386,6 +372,29 @@ void SnippetsManager::Private::insertActionSnippet()
                               Q_ARG(QString, text));
 }
 
+void SnippetsManager::Private::initializeActionCollection()
+{
+    const QVector<SnippetsInfo> infos = mModel->snippetsInfo();
+    for (const SnippetsInfo &info : infos) {
+        initializeAction(info.newName, info.keySequence, info.text);
+    }
+}
+
+void SnippetsManager::Private::initializeAction(const QString &newName, const QKeySequence &keySequence, const QString &text)
+{
+    const QString actionName = i18nc("@action", "Snippet %1", newName);
+    const QString normalizedName = QString(actionName).replace(QLatin1Char(' '), QLatin1Char('_'));
+
+    QAction *action
+        = mActionCollection->addAction(normalizedName, q);
+    connect(action, &QAction::triggered, q, [this]() {
+        insertActionSnippet();
+    });
+    action->setProperty("snippetText", text);
+    action->setText(actionName);
+    mActionCollection->setDefaultShortcut(action, keySequence);
+}
+
 void SnippetsManager::Private::updateActionCollection(const QString &oldName, const QString &newName, const QKeySequence &keySequence, const QString &text)
 {
     // remove previous action in case that the name changed
@@ -400,17 +409,7 @@ void SnippetsManager::Private::updateActionCollection(const QString &oldName, co
     }
 
     if (!newName.isEmpty()) {
-        const QString actionName = i18nc("@action", "Snippet %1", newName);
-        const QString normalizedName = QString(actionName).replace(QLatin1Char(' '), QLatin1Char('_'));
-
-        QAction *action
-            = mActionCollection->addAction(normalizedName, q);
-        connect(action, &QAction::triggered, q, [this]() {
-            insertActionSnippet();
-        });
-        action->setProperty("snippetText", text);
-        action->setText(actionName);
-        mActionCollection->setDefaultShortcut(action, keySequence);
+        initializeAction(newName, keySequence, text);
     }
 }
 
@@ -419,10 +418,10 @@ QString SnippetsManager::Private::replaceVariables(const QString &text)
     QString result = text;
     QString variableName;
     QString variableValue;
-    QMap<QString, QString> localVariables(mSavedVariables);
+    QMap<QString, QString> localVariables(SnippetsModel::instance()->savedVariables());
     int iFound = -1;
     int iEnd = -1;
-
+    QMap<QString, QString> tempLocalVariables(localVariables);
     do {
         //find the next variable by this QRegExp
         iFound = text.indexOf(QRegExp(QLatin1String("\\$[A-Za-z-_0-9\\s]*\\$")), iEnd + 1);
@@ -433,7 +432,7 @@ QString SnippetsManager::Private::replaceVariables(const QString &text)
 
             if (variableName != QLatin1String("$$")) {      // if not double-delimiter
                 if (!localVariables.contains(variableName)) {      // and not already in map
-                    QPointer<SnippetVariableDialog> dlg = new SnippetVariableDialog(variableName, &mSavedVariables, mParent);
+                    QPointer<SnippetVariableDialog> dlg = new SnippetVariableDialog(variableName, &tempLocalVariables, mParent);
                     if (dlg->exec()) {
                         if (dlg->saveVariableIsChecked()) {
                             mDirty = true;
@@ -455,147 +454,18 @@ QString SnippetsManager::Private::replaceVariables(const QString &text)
             localVariables[ variableName ] = variableValue;
         }
     } while (iFound != -1);
+    SnippetsModel::instance()->setSavedVariables(tempLocalVariables);
 
     return result;
 }
 
-QModelIndex SnippetsManager::Private::createGroup(const QString &groupName)
-{
-    mModel->insertRow(mModel->rowCount(), QModelIndex());
-    const QModelIndex groupIndex = mModel->index(mModel->rowCount() - 1, 0, QModelIndex());
-    mModel->setData(groupIndex, groupName, SnippetsModel::NameRole);
-    return groupIndex;
-}
-
-void SnippetsManager::Private::createSnippet(const QModelIndex &groupIndex, const QString &snippetName, const QString &snippetText, const QString &snippetKeySequence)
-{
-    mModel->insertRow(mModel->rowCount(groupIndex), groupIndex);
-    const QModelIndex index = mModel->index(mModel->rowCount(groupIndex) - 1, 0, groupIndex);
-
-    mModel->setData(index, snippetName, SnippetsModel::NameRole);
-    mModel->setData(index, snippetText, SnippetsModel::TextRole);
-    mModel->setData(index, snippetKeySequence, SnippetsModel::KeySequenceRole);
-
-    updateActionCollection(QString(),
-                           snippetName,
-                           QKeySequence::fromString(snippetKeySequence),
-                           snippetText);
-}
-
-void SnippetsManager::Private::load()
-{
-    const KSharedConfig::Ptr config
-        = KSharedConfig::openConfig(QStringLiteral("kmailsnippetrc"), KConfig::NoGlobals);
-
-    const KConfigGroup snippetPartGroup = config->group("SnippetPart");
-
-    const int groupCount = snippetPartGroup.readEntry("snippetGroupCount", 0);
-
-    for (int i = 0; i < groupCount; ++i) {
-        const KConfigGroup group
-            = config->group(QStringLiteral("SnippetGroup_%1").arg(i));
-
-        const QString groupName = group.readEntry("Name");
-
-        // create group
-        QModelIndex groupIndex = createGroup(groupName);
-
-        const int snippetCount = group.readEntry("snippetCount", 0);
-        for (int j = 0; j < snippetCount; ++j) {
-            const QString snippetName
-                = group.readEntry(QStringLiteral("snippetName_%1").arg(j), QString());
-
-            const QString snippetText
-                = group.readEntry(QStringLiteral("snippetText_%1").arg(j), QString());
-
-            const QString snippetKeySequence
-                = group.readEntry(QStringLiteral("snippetKeySequence_%1").arg(j), QString());
-
-            createSnippet(groupIndex, snippetName, snippetText, snippetKeySequence);
-        }
-    }
-
-    mSavedVariables.clear();
-    const KConfigGroup group = config->group("SavedVariablesPart");
-    const int variablesCount = group.readEntry("variablesCount", 0);
-
-    for (int i = 0; i < variablesCount; ++i) {
-        const QString variableKey
-            = group.readEntry(QStringLiteral("variableName_%1").arg(i), QString());
-
-        const QString variableValue
-            = group.readEntry(QStringLiteral("variableValue_%1").arg(i), QString());
-
-        mSavedVariables.insert(variableKey, variableValue);
-    }
-}
-
 void SnippetsManager::Private::save()
 {
     if (!mDirty) {
         return;
     }
 
-    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kmailsnippetrc"), KConfig::NoGlobals);
-
-    // clear everything
-    for (const QString &group : config->groupList()) {
-        config->deleteGroup(group);
-    }
-
-    // write number of snippet groups
-    KConfigGroup group = config->group("SnippetPart");
-
-    const int groupCount = mModel->rowCount();
-    group.writeEntry("snippetGroupCount", groupCount);
-
-    for (int i = 0; i < groupCount; ++i) {
-        const QModelIndex groupIndex = mModel->index(i, 0, QModelIndex());
-        const QString groupName = groupIndex.data(SnippetsModel::NameRole).toString();
-
-        KConfigGroup group = config->group(QStringLiteral("SnippetGroup_%1").arg(i));
-        group.writeEntry("Name", groupName);
-
-        const int snippetCount = mModel->rowCount(groupIndex);
-
-        group.writeEntry("snippetCount", snippetCount);
-        for (int j = 0; j < snippetCount; ++j) {
-            const QModelIndex index = mModel->index(j, 0, groupIndex);
-
-            const QString snippetName = index.data(SnippetsModel::NameRole).toString();
-            if (!snippetName.isEmpty()) {
-                const QString snippetText = index.data(SnippetsModel::TextRole).toString();
-                const QString snippetKeySequence = index.data(SnippetsModel::KeySequenceRole).toString();
-
-                group.writeEntry(QStringLiteral("snippetName_%1").arg(j), snippetName);
-                if (!snippetText.isEmpty()) {
-                    group.writeEntry(QStringLiteral("snippetText_%1").arg(j), snippetText);
-                }
-                if (!snippetKeySequence.isEmpty()) {
-                    group.writeEntry(QStringLiteral("snippetKeySequence_%1").arg(j),
-                                     snippetKeySequence);
-                }
-            }
-        }
-    }
-
-    {
-        KConfigGroup group = config->group("SavedVariablesPart");
-
-        const int variablesCount = mSavedVariables.count();
-        group.writeEntry("variablesCount", variablesCount);
-
-        int counter = 0;
-        QMap<QString, QString>::const_iterator it = mSavedVariables.cbegin();
-        const QMap<QString, QString>::const_iterator itEnd = mSavedVariables.cend();
-        for (; it != itEnd; ++it) {
-            group.writeEntry(QStringLiteral("variableName_%1").arg(counter), it.key());
-            group.writeEntry(QStringLiteral("variableValue_%1").arg(counter), it.value());
-            counter++;
-        }
-    }
-
-    config->sync();
+    SnippetsModel::instance()->save();
     mDirty = false;
 }
 
@@ -603,17 +473,22 @@ SnippetsManager::SnippetsManager(KActionCollection *actionCollection, QObject *p
     : QObject(parent)
     , d(new Private(this, widget))
 {
-    d->mModel = new SnippetsModel(this);
+    d->mModel = SnippetsModel::instance();
+    connect(d->mModel, &SnippetsModel::updateActionCollection, this, [this](const QString &oldName, const QString &newName, const QKeySequence &keySequence, const QString &text) {
+        d->updateActionCollection(oldName, newName, keySequence, text);
+    });
     d->mSelectionModel = new QItemSelectionModel(d->mModel);
     d->mActionCollection = actionCollection;
 
     d->mAddSnippetAction = new QAction(i18n("Add Snippet..."), this);
+    d->mAddSnippetAction->setIcon(QIcon::fromTheme(QStringLiteral("list-add")));
     d->mEditSnippetAction = new QAction(i18n("Edit Snippet..."), this);
     d->mEditSnippetAction->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
     d->mDeleteSnippetAction = new QAction(i18n("Remove Snippet"), this);
     d->mDeleteSnippetAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-delete")));
 
     d->mAddSnippetGroupAction = new QAction(i18n("Add Group..."), this);
+    d->mAddSnippetGroupAction->setIcon(QIcon::fromTheme(QStringLiteral("list-add")));
     d->mEditSnippetGroupAction = new QAction(i18n("Rename Group..."), this);
     d->mEditSnippetGroupAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
     d->mDeleteSnippetGroupAction = new QAction(i18n("Remove Group"), this);
@@ -656,9 +531,8 @@ SnippetsManager::SnippetsManager(KActionCollection *actionCollection, QObject *p
         d->insertSelectedSnippet();
     });
 
+    d->initializeActionCollection();
     d->selectionChanged();
-
-    d->load();
 }
 
 SnippetsManager::~SnippetsManager()
