@@ -25,14 +25,15 @@
 #include <QMimeData>
 #include <QDataStream>
 #include <QStringList>
-#include <QDebug>
+#include <KSharedConfig>
+#include <KConfigGroup>
 
 using namespace MailCommon;
 
 class MailCommon::SnippetItem
 {
 public:
-    SnippetItem(bool isGroup = false, SnippetItem *parent = nullptr);
+    explicit SnippetItem(bool isGroup = false, SnippetItem *parent = nullptr);
     ~SnippetItem();
 
     bool isGroup() const;
@@ -55,9 +56,9 @@ public:
 
 private:
     QList<SnippetItem *> mChildItems;
-    SnippetItem *mParentItem;
+    SnippetItem *mParentItem = nullptr;
 
-    bool mIsGroup;
+    bool mIsGroup = false;
     QString mName;
     QString mText;
     QString mKeySequence;
@@ -145,10 +146,17 @@ int SnippetItem::row() const
     return 0;
 }
 
+SnippetsModel *SnippetsModel::instance()
+{
+    static SnippetsModel s_self;
+    return &s_self;
+}
+
 SnippetsModel::SnippetsModel(QObject *parent)
     : QAbstractItemModel(parent)
 {
     mRootItem = new SnippetItem(true);
+    load();
 }
 
 SnippetsModel::~SnippetsModel()
@@ -175,20 +183,16 @@ bool SnippetsModel::setData(const QModelIndex &index, const QVariant &value, int
         item->setName(value.toString());
         Q_EMIT dataChanged(index, index);
         return true;
-        break;
     case TextRole:
         item->setText(value.toString());
         Q_EMIT dataChanged(index, index);
         return true;
-        break;
     case KeySequenceRole:
         item->setKeySequence(value.toString());
         Q_EMIT dataChanged(index, index);
         return true;
-        break;
     default:
         return false;
-        break;
     }
 
     return false;
@@ -205,19 +209,14 @@ QVariant SnippetsModel::data(const QModelIndex &index, int role) const
     switch (role) {
     case Qt::DisplayRole:
         return item->name();
-        break;
     case IsGroupRole:
         return item->isGroup();
-        break;
     case NameRole:
         return item->name();
-        break;
     case TextRole:
         return item->text();
-        break;
     case KeySequenceRole:
         return item->keySequence();
-        break;
     }
 
     return QVariant();
@@ -243,7 +242,7 @@ QModelIndex SnippetsModel::index(int row, int column, const QModelIndex &parent)
         return QModelIndex();
     }
 
-    SnippetItem *parentItem;
+    SnippetItem *parentItem = nullptr;
 
     if (!parent.isValid()) {
         parentItem = mRootItem;
@@ -277,7 +276,7 @@ QModelIndex SnippetsModel::parent(const QModelIndex &index) const
 
 int SnippetsModel::rowCount(const QModelIndex &parent) const
 {
-    SnippetItem *parentItem;
+    SnippetItem *parentItem = nullptr;
     if (parent.column() > 0) {
         return 0;
     }
@@ -293,7 +292,7 @@ int SnippetsModel::rowCount(const QModelIndex &parent) const
 
 bool SnippetsModel::insertRows(int row, int count, const QModelIndex &parent)
 {
-    SnippetItem *parentItem;
+    SnippetItem *parentItem = nullptr;
 
     if (!parent.isValid()) {
         parentItem = mRootItem;
@@ -313,7 +312,7 @@ bool SnippetsModel::insertRows(int row, int count, const QModelIndex &parent)
 
 bool SnippetsModel::removeRows(int row, int count, const QModelIndex &parent)
 {
-    SnippetItem *parentItem;
+    SnippetItem *parentItem = nullptr;
 
     if (!parent.isValid()) {
         parentItem = mRootItem;
@@ -384,7 +383,7 @@ bool SnippetsModel::dropMimeData(const QMimeData *data, Qt::DropAction action, i
         if (item->isGroup()) {
             Q_EMIT addNewDndSnippset(encodedData);
         } else {
-            if (KMessageBox::Yes == KMessageBox::questionYesNo(nullptr, i18n("Do you want to update snippet?"), i18n("Rename snippet"))) {
+            if (KMessageBox::Yes == KMessageBox::questionYesNo(nullptr, i18n("Do you want to update snippet?"), i18n("Update snippet"))) {
                 item->setText(encodedData);
             }
         }
@@ -438,12 +437,171 @@ Qt::DropActions SnippetsModel::supportedDropActions() const
     return Qt::CopyAction | Qt::MoveAction;
 }
 
-QHash<int, QByteArray> SnippetsModel::roleNames() const
+QModelIndex SnippetsModel::createGroup(const QString &groupName)
 {
-    static QHash<int, QByteArray> names;
-    if (names.isEmpty()) {
-        names.insert(IsGroupRole, QByteArrayLiteral("isSnippetGroup"));
+    insertRow(rowCount(), QModelIndex());
+    const QModelIndex groupIndex = index(rowCount() - 1, 0, QModelIndex());
+    setData(groupIndex, groupName, SnippetsModel::NameRole);
+    return groupIndex;
+}
+
+void SnippetsModel::load()
+{
+    const KSharedConfig::Ptr config
+        = KSharedConfig::openConfig(QStringLiteral("kmailsnippetrc"), KConfig::NoGlobals);
+
+    const KConfigGroup snippetPartGroup = config->group("SnippetPart");
+
+    const int groupCount = snippetPartGroup.readEntry("snippetGroupCount", 0);
+
+    for (int i = 0; i < groupCount; ++i) {
+        const KConfigGroup group
+            = config->group(QStringLiteral("SnippetGroup_%1").arg(i));
+
+        const QString groupName = group.readEntry("Name");
+
+        // create group
+        const QModelIndex groupIndex = createGroup(groupName);
+
+        const int snippetCount = group.readEntry("snippetCount", 0);
+        for (int j = 0; j < snippetCount; ++j) {
+            const QString snippetName
+                = group.readEntry(QStringLiteral("snippetName_%1").arg(j), QString());
+
+            const QString snippetText
+                = group.readEntry(QStringLiteral("snippetText_%1").arg(j), QString());
+
+            const QString snippetKeySequence
+                = group.readEntry(QStringLiteral("snippetKeySequence_%1").arg(j), QString());
+
+            createSnippet(groupIndex, snippetName, snippetText, snippetKeySequence);
+        }
     }
 
-    return names;
+    const KConfigGroup group = config->group("SavedVariablesPart");
+    const int variablesCount = group.readEntry("variablesCount", 0);
+
+    for (int i = 0; i < variablesCount; ++i) {
+        const QString variableKey
+            = group.readEntry(QStringLiteral("variableName_%1").arg(i), QString());
+
+        const QString variableValue
+            = group.readEntry(QStringLiteral("variableValue_%1").arg(i), QString());
+
+        mSavedVariables.insert(variableKey, variableValue);
+    }
+}
+
+void SnippetsModel::createSnippet(const QModelIndex &groupIndex, const QString &snippetName, const QString &snippetText, const QString &snippetKeySequence)
+{
+    insertRow(rowCount(groupIndex), groupIndex);
+    const QModelIndex modelIndex = index(rowCount(groupIndex) - 1, 0, groupIndex);
+
+    setData(modelIndex, snippetName, SnippetsModel::NameRole);
+    setData(modelIndex, snippetText, SnippetsModel::TextRole);
+    setData(modelIndex, snippetKeySequence, SnippetsModel::KeySequenceRole);
+
+    Q_EMIT updateActionCollection(QString(),
+                                  snippetName,
+                                  QKeySequence::fromString(snippetKeySequence),
+                                  snippetText);
+}
+
+void SnippetsModel::setSavedVariables(const QMap<QString, QString> &savedVariables)
+{
+    mSavedVariables = savedVariables;
+}
+
+QVector<SnippetsInfo> SnippetsModel::snippetsInfo() const
+{
+    QVector<SnippetsInfo> infos;
+    const int groupCount = rowCount();
+
+    for (int i = 0; i < groupCount; ++i) {
+        const QModelIndex groupIndex = index(i, 0, QModelIndex());
+        const int snippetCount = rowCount(groupIndex);
+        for (int j = 0; j < snippetCount; ++j) {
+            SnippetsInfo info;
+            const QModelIndex modelIndex = index(j, 0, groupIndex);
+
+            const QString snippetName = modelIndex.data(SnippetsModel::NameRole).toString();
+            if (!snippetName.isEmpty()) {
+                const QString snippetText = modelIndex.data(SnippetsModel::TextRole).toString();
+                const QString snippetKeySequence = modelIndex.data(SnippetsModel::KeySequenceRole).toString();
+                info.text = snippetText;
+                info.newName = snippetName;
+                info.keySequence = QKeySequence::fromString(snippetKeySequence);
+                infos.append(info);
+            }
+        }
+    }
+    return infos;
+}
+
+QMap<QString, QString> SnippetsModel::savedVariables() const
+{
+    return mSavedVariables;
+}
+
+void SnippetsModel::save()
+{
+    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kmailsnippetrc"), KConfig::NoGlobals);
+
+    // clear everything
+    for (const QString &group : config->groupList()) {
+        config->deleteGroup(group);
+    }
+
+    // write number of snippet groups
+    KConfigGroup group = config->group("SnippetPart");
+
+    const int groupCount = rowCount();
+    group.writeEntry("snippetGroupCount", groupCount);
+
+    for (int i = 0; i < groupCount; ++i) {
+        const QModelIndex groupIndex = index(i, 0, QModelIndex());
+        const QString groupName = groupIndex.data(SnippetsModel::NameRole).toString();
+
+        KConfigGroup group = config->group(QStringLiteral("SnippetGroup_%1").arg(i));
+        group.writeEntry("Name", groupName);
+
+        const int snippetCount = rowCount(groupIndex);
+
+        group.writeEntry("snippetCount", snippetCount);
+        for (int j = 0; j < snippetCount; ++j) {
+            const QModelIndex modelIndex = index(j, 0, groupIndex);
+
+            const QString snippetName = modelIndex.data(SnippetsModel::NameRole).toString();
+            if (!snippetName.isEmpty()) {
+                const QString snippetText = modelIndex.data(SnippetsModel::TextRole).toString();
+                const QString snippetKeySequence = modelIndex.data(SnippetsModel::KeySequenceRole).toString();
+
+                group.writeEntry(QStringLiteral("snippetName_%1").arg(j), snippetName);
+                if (!snippetText.isEmpty()) {
+                    group.writeEntry(QStringLiteral("snippetText_%1").arg(j), snippetText);
+                }
+                if (!snippetKeySequence.isEmpty()) {
+                    group.writeEntry(QStringLiteral("snippetKeySequence_%1").arg(j),
+                                     snippetKeySequence);
+                }
+            }
+        }
+    }
+
+    {
+        KConfigGroup group = config->group("SavedVariablesPart");
+
+        const int variablesCount = mSavedVariables.count();
+        group.writeEntry("variablesCount", variablesCount);
+
+        int counter = 0;
+        QMap<QString, QString>::const_iterator it = mSavedVariables.cbegin();
+        const QMap<QString, QString>::const_iterator itEnd = mSavedVariables.cend();
+        for (; it != itEnd; ++it) {
+            group.writeEntry(QStringLiteral("variableName_%1").arg(counter), it.key());
+            group.writeEntry(QStringLiteral("variableValue_%1").arg(counter), it.value());
+            counter++;
+        }
+    }
+    config->sync();
 }
