@@ -22,10 +22,17 @@
 #define MAILCOMMON_SNIPPETSMODEL_P_H
 
 #include <QAbstractItemModel>
-
+#include <QKeySequence>
 namespace MailCommon {
 class SnippetItem;
 
+struct SnippetsInfo
+{
+    QString newName;
+    QKeySequence keySequence;
+    QString text;
+};
+
 class SnippetsModel : public QAbstractItemModel
 {
     Q_OBJECT
@@ -37,30 +44,39 @@ public:
         KeySequenceRole                 ///< The key sequence to activate a snippet
     };
 
+    static SnippetsModel *instance();
+
     explicit SnippetsModel(QObject *parent = nullptr);
     ~SnippetsModel() override;
 
-    bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
+    Q_REQUIRED_RESULT bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
 
-    QVariant data(const QModelIndex &index, int role) const override;
+    Q_REQUIRED_RESULT QVariant data(const QModelIndex &index, int role) const override;
 
-    Qt::ItemFlags flags(const QModelIndex &index) const override;
+    Q_REQUIRED_RESULT Qt::ItemFlags flags(const QModelIndex &index) const override;
 
-    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
+    Q_REQUIRED_RESULT QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
 
-    QModelIndex parent(const QModelIndex &index) const override;
+    Q_REQUIRED_RESULT QModelIndex parent(const QModelIndex &index) const override;
 
-    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    Q_REQUIRED_RESULT int rowCount(const QModelIndex &parent = QModelIndex()) const override;
 
-    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
+    Q_REQUIRED_RESULT int columnCount(const QModelIndex &parent = QModelIndex()) const override;
 
-    QStringList mimeTypes() const override;
+    Q_REQUIRED_RESULT QStringList mimeTypes() const override;
 
-    QMimeData *mimeData(const QModelIndexList &indexes) const override;
+    Q_REQUIRED_RESULT QMimeData *mimeData(const QModelIndexList &indexes) const override;
 
-    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
+    Q_REQUIRED_RESULT bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
 
-    Qt::DropActions supportedDropActions() const override;
+    Q_REQUIRED_RESULT Qt::DropActions supportedDropActions() const override;
+
+    void save();
+
+    Q_REQUIRED_RESULT QMap<QString, QString> savedVariables() const;
+    void setSavedVariables(const QMap<QString, QString> &savedVariables);
+
+    Q_REQUIRED_RESULT QVector<SnippetsInfo> snippetsInfo() const;
 
 protected:
     bool insertRows(int row, int count, const QModelIndex &parent = QModelIndex()) override;
@@ -70,9 +86,16 @@ protected:
 Q_SIGNALS:
     void dndDone();
     void addNewDndSnippset(const QString &);
+    void updateActionCollection(const QString &oldName, const QString &newName, const QKeySequence &keySequence, const QString &text);
+
 private:
+    void load();
+    QModelIndex createGroup(const QString &groupName);
+    void createSnippet(const QModelIndex &groupIndex, const QString &snippetName, const QString &snippetText, const QString &snippetKeySequence);
     SnippetItem *mRootItem = nullptr;
+    QMap<QString, QString> mSavedVariables;
 };
 }
+Q_DECLARE_TYPEINFO(MailCommon::SnippetsInfo, Q_MOVABLE_TYPE);
 
 #endif
